#ifndef _IN6_H_
#define	_IN6_H_

#include <netinet/in.h>
#include <netinet/in6_machtypes.h>

/* IPv6 protocols */
#define IPPROTO_HOPBYHOP	0		/* Hop-by-hop options */
#define IPPROTO_IPV6            41		/* IPv6 header */
#define IPPROTO_ROUTING		43		/* Routing header */
#define IPPROTO_FRAG		44		/* Fragmentation header */
#define IPPROTO_ESP		50		/* Encap. security payload */
#define IPPROTO_AUTH		51		/* Authentication header */
#define IPPROTO_ICMPV6		58
#define	IPPROTO_NONEXTHDR	59
#define	IPPROTO_DESTNODE	60		/* Destination Node header */

/*
 * Interface token lengths (bits)
 */

#define IPV6_LOOPBACK_TOKEN_LENGTH	 0
#define IPV6_ETHERNET_TOKEN_LENGTH	48
#define IPV6_FDDI_TOKEN_LENGTH		48
#define IPV6_IPV4_TOKEN_LENGTH		48

/*
 * Define the extra set/getsockopts for the IPv6 level.
 */
#define IP_FLOWLABEL		14	      /* int; flow label in net byte order */
#define IP_RXHDR		15	      /* int; include rx info in raw recv */

#define IPV6_FLOWINFO_FLOWLABEL		IN6__MK3_MSB32_VALUE(0, 0xFF, 0xFF, 0xFF)
#define	IPV6_FLOWINFO_PRIORITY		IN6__MK0_MSB32_VALUE(0x0F)
#define	IPV6_FLOWINFO_SRFLAG		IN6__MK0_MSB32_VALUE(0x30)
#define	IPV6_SRFLAG_NONE		IN6__MK0_MSB32_VALUE(0x00)
#define	IPV6_SRFLAG_STRICT		IN6__MK0_MSB32_VALUE(0x10)
#define	IPV6_SRFLAG_LOOSE		IN6__MK0_MSB32_VALUE(0x20)

#define	IPV6_PRIORITY_UNCHARACTERIZED	IN6__MK0_MSB32_VALUE(0x00)
#define	IPV6_PRIORITY_FILLER		IN6__MK0_MSB32_VALUE(0x01)
#define	IPV6_PRIORITY_UNATTENDED	IN6__MK0_MSB32_VALUE(0x02)
#define	IPV6_PRIORITY_RESERVED1		IN6__MK0_MSB32_VALUE(0x03)
#define	IPV6_PRIORITY_BULK		IN6__MK0_MSB32_VALUE(0x04)
#define	IPV6_PRIORITY_RESERVED2		IN6__MK0_MSB32_VALUE(0x05)
#define	IPV6_PRIORITY_INTERACTIVE	IN6__MK0_MSB32_VALUE(0x06)
#define	IPV6_PRIORITY_CONTROL		IN6__MK0_MSB32_VALUE(0x07)
#define	IPV6_PRIORITY_8			IN6__MK0_MSB32_VALUE(0x08)
#define	IPV6_PRIORITY_9			IN6__MK0_MSB32_VALUE(0x09)
#define	IPV6_PRIORITY_10		IN6__MK0_MSB32_VALUE(0x0A)
#define	IPV6_PRIORITY_11		IN6__MK0_MSB32_VALUE(0x0B)
#define	IPV6_PRIORITY_12		IN6__MK0_MSB32_VALUE(0x0C)
#define	IPV6_PRIORITY_13		IN6__MK0_MSB32_VALUE(0x0D)
#define	IPV6_PRIORITY_14		IN6__MK0_MSB32_VALUE(0x0E)
#define	IPV6_PRIORITY_15		IN6__MK0_MSB32_VALUE(0x0F)

/*
 * IPv6 level socket options
 */
#define	IPV6_ADDRFORM		100	/* int; AF_INET or AF_INET6 */
#define	IPV6_RECVSRCRT		101
#define	IPV6_RECVIF		102
#define	IPV6_SENDIF		103
#define IPV6_UNICAST_HOPS	104	/* int; unicast hoplimit (0..255) */
#define IPV6_MULTICAST_HOPS	105	/* u_int; multicast hoplimit (0..255) */
#define	IPV6_MULTICAST_IF	106	/* struct in6_addr; mcast interface */ 
#define	IPV6_MULTICAST_LOOP	107	/* u_int; recv own multicast (0 or 1) */
#define	IPV6_ADD_MEMBERSHIP	108	/* struct ipv6_mreq; join group */
#define	IPV6_DROP_MEMBERSHIP	109	/* struct ipv6_mreq; leave group */
#define IPV6_RXINFO		110	/* int; recv ip6_pktinfo */

#define IPV6_DEFAULT_MULTICAST_HOPS	1 /* normally limit mcasts to 1 hop */
#define IPV6_DEFAULT_MULTICAST_LOOP	1 /* normally hear sends if a member */
#define IPV6_MAX_MEMBERSHIPS		20 /* per socket */

#define	IPV6_SR_MAXADDR			24

/*
 *   Define the IPv6 address.
 */
struct in6_addr {
    union {
	u_char sa6_addr[16];
#define	s6_addr		s6_un.sa6_addr
	u_int16_t sa6_waddr[8];
#define	s6_waddr	s6_un.sa6_waddr
	u_int32_t sa6_laddr[4];
#define	s6_laddr	s6_un.sa6_laddr
#ifdef IN6_HAS_64BIT_INTTYPE
	u_int64_t sa6_qaddr[2];
#define	s6_qaddr	s6_un.sa6_qaddr
#endif
    } s6_un;
};

/*
 * Maximum length of IPv6 address as text string
 */

#define INET6_ADDRSTRLEN	46

/*
 * Structure for add/drop multicast group membership
 */

struct ipv6_mreq {
    struct in6_addr ipv6mr_multiaddr;
    struct in6_addr ipv6mr_interface;
};

/*
 * Define the macros to manipulate IPv6 addresses
 */
#ifdef IN6_HAS_64BIT_INTTYPE
#define IN6_ARE_ADDR_EQUAL(a,b)		((a)->s6_qaddr[1] == (b)->s6_qaddr[1] && \
					 (a)->s6_qaddr[0] == (b)->s6_qaddr[0])
#define IN6_IS_ADDR_HOSTMASK(a)		(((a)->s6_qaddr[0] & (a)->s6_qaddr[1]) == ~0UL)
#define	IN6_IS_ADDR_UNSPECIFIED(a)	(((a)->s6_qaddr[0] | (a)->s6_qaddr[1]) == 0UL)
#define	IN6_SET_ADDR_UNSPECIFIED(a)	((a)->s6_qaddr[1] = ((a)->s6_qaddr[0] = 0UL))
#define	IN6_SET_ADDR_HOSTMASK(a)	((a)->s6_qaddr[1] = ((a)->s6_qaddr[0] = ~0UL))
#define IN6_SET_LINKLOCAL_NETMASK(a)	((a)->s6_qaddr[0] = ~0UL, \
					 (a)->s6_qaddr[1] = IN6__MK0_MSW64_VALUE(0xFFFF))
#define	IN6_SET_ADDR_PREFIX(a,b,c)	((a)->s6_qaddr[0] = (b)->s6_qaddr[0] & (c)->s6_qaddr[0], \
					 (a)->s6_qaddr[1] = (b)->s6_qaddr[1] & (c)->s6_qaddr[1])
#define	IN6_SET_ADDR_LOOPBACK(a)	((a)->s6_qaddr[0] = 0UL, \
					 (a)->s6_qaddr[1] = IN6__MK7_MSB64_VALUE(0,0,0,0,0,0,0,1))
#define	IN6_ARE_ADDR_SAMEPREFIX(a,b,m)	(((a)->s6_qaddr[0] & (m)->s6_qaddr[0]) == (b)->s6_qaddr[0] \
					 && ((a)->s6_qaddr[1] & (m)->s6_qaddr[1]) == (b)->s6_qaddr[1])
#define	IN6_IS_ADDR_V4COMPAT(a)		((a)->s6_qaddr[0] == 0UL && (a)->s6_laddr[2] == 0)
#define IN6_IS_ADDR_V4MAPPED(a)		((a)->s6_qaddr[0] == 0UL && \
					 (a)->s6_laddr[2] == IN6__MK1_MSW32_VALUE(0, 0xFFFFU))
#define IN6_SET_ADDR_V4MAPPED(a,b)	((a)->s6_qaddr[0] = 0, \
					 (a)->s6_laddr[2] = IN6__MK1_MSW32_VALUE(0, 0xFFFFU), \
					 (a)->s6_laddr[3] = (b))
#define IN6_SET_ADDR_V4COMPAT(a,b)	((a)->s6_qaddr[0] = 0, \
					 (a)->s6_laddr[2] = 0, \
					 (a)->s6_laddr[3] = (b))
#define IN6_JOIN_PREFIX_TOKEN(a,p,t)	((a)->s6_qaddr[0] = \
					  (p)->s6_qaddr[0] | (t)->s6_qaddr[0], \
					 (a)->s6_qaddr[1] = \
					  (p)->s6_qaddr[1] | (t)->s6_qaddr[1])
#else
#define IN6_ARE_ADDR_EQUAL(a,b)		((a)->s6_laddr[3] == (b)->s6_laddr[3] && \
					 (a)->s6_laddr[0] == (b)->s6_laddr[0] && \
					 (a)->s6_laddr[2] == (b)->s6_laddr[2] && \
					 (a)->s6_laddr[1] == (b)->s6_laddr[1])
#define IN6_IS_ADDR_HOSTMASK(a)		((a)->s6_laddr[3] == ~0UL && \
					 (a)->s6_laddr[0] == ~0UL && \
					 (a)->s6_laddr[2] == ~0UL && \
					 (a)->s6_laddr[1] == ~0UL)
#define IN6_IS_ADDR_UNSPECIFIED(a)	((a)->s6_laddr[3] == 0UL && \
					 (a)->s6_laddr[0] == 0UL && \
					 (a)->s6_laddr[2] == 0UL && \
					 (a)->s6_laddr[1] == 0UL)
#define	IN6_SET_ADDR_UNSPECIFIED(a)	((a)->s6_laddr[3] = ((a)->s6_laddr[2] = ((a)->s6_laddr[1] = ((a)->s6_laddr[0] = 0UL))))
#define	IN6_SET_ADDR_HOSTMASK(a)	((a)->s6_laddr[3] = ((a)->s6_laddr[2] = ((a)->s6_laddr[1] = ((a)->s6_laddr[0] = ~0UL))))
#define IN6_SET_LINKLOCAL_NETMASK(a)	((a)->s6_laddr[0] = ((a)->s6_laddr[1] = ~0UL), \
					 (a)->s6_laddr[2] = IN6__MK0_MSW32_VALUE(0xFFFFU), \
					 (a)->s6_laddr[3] = 0UL)
#define	IN6_SET_ADDR_PREFIX(a,b,c)	((a)->s6_laddr[0] = (b)->s6_laddr[0] & (c)->s6_laddr[0], \
					 (a)->s6_laddr[1] = (b)->s6_laddr[1] & (c)->s6_laddr[1], \
					 (a)->s6_laddr[2] = (b)->s6_laddr[2] & (c)->s6_laddr[2], \
					 (a)->s6_laddr[3] = (b)->s6_laddr[3] & (c)->s6_laddr[3])
#define	IN6_SET_ADDR_LOOPBACK(a)	((a)->s6_laddr[0] = ((a)->s6_laddr[1] = ((a)->s6_laddr[2] = 0UL)), \
					 (a)->s6_laddr[3] = IN6__MK3_MSB32_VALUE(0,0,0,1))
#define IN6_IS_ADDR_V4COMPAT(a)		((a)->s6_laddr[0] == 0UL \
					 && (a)->s6_laddr[1] == 0UL \
					 && (a)->s6_laddr[2] == 0UL)
#define IN6_IS_ADDR_V4MAPPED(a)		((a)->s6_laddr[0] == 0UL && (a)->s6_laddr[1] == 0UL && \
					 (a)->s6_laddr[2] == IN6__MK1_MSW32_VALUE(0, 0xFFFFU))
#define	IN6_ARE_ADDR_SAMEPREFIX(a,b,m)	(((a)->s6_laddr[0] & (m)->s6_laddr[0]) == (b)->s6_laddr[0] \
					 && ((a)->s6_laddr[1] & (m)->s6_laddr[1]) == (b)->s6_laddr[1] \
					 && ((a)->s6_laddr[2] & (m)->s6_laddr[2]) == (b)->s6_laddr[2] \
					 && ((a)->s6_laddr[3] & (m)->s6_laddr[3]) == (b)->s6_laddr[3])
#define IN6_SET_ADDR_V4MAPPED(a,b)	((a)->s6_laddr[0] = 0, (a)->s6_laddr[1] = 0, \
					 (a)->s6_laddr[2] = IN6__MK1_MSW32_VALUE(0, 0xFFFFU), \
					 (a)->s6_laddr[3] = (b))
#define IN6_SET_ADDR_V4COMPAT(a,b)	((a)->s6_laddr[0] = 0, (a)->s6_laddr[1] = 0, \
					 (a)->s6_laddr[2] = 0, \
					 (a)->s6_laddr[3] = (b))
#define IN6_JOIN_PREFIX_TOKEN(a,p,t)	((a)->s6_laddr[0] = \
					  (p)->s6_laddr[0] | (t)->s6_laddr[0], \
					 (a)->s6_laddr[1] = \
					  (p)->s6_laddr[1] | (t)->s6_laddr[1], \
					 (a)->s6_laddr[2] = \
					  (p)->s6_laddr[2] | (t)->s6_laddr[2], \
					 (a)->s6_laddr[3] = \
					  (p)->s6_laddr[3] | (t)->s6_laddr[3])
#endif

#define IN6_IS_ADDR_v4v6(a)		IN6_IS_ADDR_V4COMPAT(a)
#define IN6_IS_ADDR_v4only(a)		IN6_IS_ADDR_V4MAPPED(a)

#define	IN6_EXTRACT_V4ADDR(a)		((a)->s6_laddr[3])

#define IN6_IS_ADDR_LINKLOCAL(a)	(((a)->s6_laddr[0] & IN6__MK1_MSB32_VALUE(0xFF, 0xC0)) == IN6__MK1_MSB32_VALUE(0xFE, 0x80))
#define IN6_IS_ADDR_SITELOCAL(a)	(((a)->s6_laddr[0] & IN6__MK1_MSB32_VALUE(0xFF, 0xC0)) == IN6__MK1_MSB32_VALUE(0xFE, 0xC0))
#define IN6_IS_ADDR_MULTICAST(a)	(((a)->s6_laddr[0] & IN6__MK0_MSB32_VALUE(0xFF)) == IN6__MK0_MSB32_VALUE(0xFF))
#define IN6_IS_ADDR_MCNODELOCAL(a)	(((a)->s6_laddr[0] & IN6__MK1_MSB32_VALUE(0xFF, 0x0F)) == IN6__MK1_MSB32_VALUE(0xFF, IN6_MCSCOPE_NODELOCAL))
#define IN6_IS_ADDR_MCLINKLOCAL(a)	(((a)->s6_laddr[0] & IN6__MK1_MSB32_VALUE(0xFF, 0x0F)) == IN6__MK1_MSB32_VALUE(0xFF, IN6_MCSCOPE_LINKLOCAL))
#define IN6_IS_ADDR_MCSITELOCAL(a)	(((a)->s6_laddr[0] & IN6__MK1_MSB32_VALUE(0xFF, 0x0F)) == IN6__MK1_MSB32_VALUE(0xFF, IN6_MCSCOPE_SITELOCAL))
#define IN6_IS_ADDR_MCORGLOCAL(a)	(((a)->s6_laddr[0] & IN6__MK1_MSB32_VALUE(0xFF, 0x0F)) == IN6__MK1_MSB32_VALUE(0xFF, IN6_MCSCOPE_ORGLOCAL))
#define IN6_IS_ADDR_MCGLOBAL(a)		(((a)->s6_laddr[0] & IN6__MK1_MSB32_VALUE(0xFF, 0x0F)) == IN6__MK1_MSB32_VALUE(0xFF, IN6_MCSCOPE_GLOBAL))
#define IN6_SET_LINKLOCAL_PREFIX(a)	((a)->s6_laddr[0] = IN6__MK1_MSB32_VALUE(0xFE, 0x80), \
					 (a)->s6_laddr[1] = ((a)->s6_laddr[2] = ((a)->s6_laddr[3] = 0)))

#ifdef IN6_HAS_64BIT_INTTYPE
#define IN6_IS_UNSPEC(a)            (((a).s6_qaddr[0] == 0UL) && ((a).s6_qaddr[1] == 0UL))
#define IN6_SET_UNSPEC(a)           ((a).s6_qaddr[0] = 0UL, (a).s6_qaddr[1] = 0UL)
#define IN6_IS_LOOPBACK(a)          (((a).s6_qaddr[0] == 0UL) && \
                                     ((a).s6_qaddr[1] == (0x01 << IN6_LSB64_SHIFT)))
#define IN6_IS_LINK_LOCAL(a)        (((a).s6_qaddr[0] & 0xC0FFl) == 0x80FEl)
#define IN6_IS_SITE_LOCAL(a)        (((a).s6_qaddr[0] & 0xC0FFl) == 0xC0FEl)
#define IN6_SET_SITELOCAL_PREFIX(a) ((a).s6_qaddr[0] = 0x000000000000C0FE, \
				     (a).s6_qaddr[1] = 0x0000000000000000)
#define IN6_SET_HOSTMASK(a)         ((a).s6_qaddr[0] = 0xFFFFFFFFFFFFFFFF, \
                                     (a).s6_qaddr[1] = 0xFFFFFFFFFFFFFFFF)
#define IN6_GET_PREFIX(a,b,mask)    ((a).s6_qaddr[0] = (b).s6_qaddr[0] & (mask).s6_qaddr[0], \
                                     (a).s6_qaddr[1] = (b).s6_qaddr[1] & (mask).s6_qaddr[1])
#define IN6_IS_MULTICAST(a)         (((a).s6_qaddr[0] & 0xFFl) == 0xFFl)
#define IN6_MULTICAST_SCOPE(a)      (((a).s6_qaddr[0] >> IN6_MSB64_OFFSET(1)) & 0x0F)
#define IN6_MULTICAST_FLAGS(a)      ((((a).s6_qaddr[0] & 0xF000l) >> 12) & 0x0F)
#define IN6_SET_ALLNODES_GROUP(a)   ((a).s6_qaddr[0] = 0x00000000000002FF, \
                                     (a).s6_qaddr[1] = 0x0100000000000000)
#define IN6_SET_ALLRTRS_GROUP(a)    ((a).s6_qaddr[0] = 0x00000000000002FF, \
                                     (a).s6_qaddr[1] = 0x0200000000000000)
#define IN6_EQUAL_ADDR(a,b)         (((a).s6_qaddr[1] == (b).s6_qaddr[1]) && \
                                     ((a).s6_qaddr[0] == (b).s6_qaddr[0]))
#ifdef nevermore
#define IN6_IS_SIT_ADDR(a)          (((a).s6_qaddr[0] == 0l) && \
                                     (!((a).s6_qaddr[1] & 0xFFFFFFFF) || \
                                      ((a).s6_qaddr[1] & 0xFFFFFFFF) == 0xFFFF0000))
#define IN6_IS_SIT_v4v6(a)          (((a).s6_qaddr[0] == 0l) && \
				     (!((a).s6_qaddr[1] & 0xFFFFFFFF)))
#define IN6_IS_SIT_v4only(a)        (((a).s6_qaddr[0] == 0l) && \
                                     ((a).s6_qaddr[1] & 0xFFFFFFFF) == 0xFFFF0000)
#define IN6_SIT_SET_v4only(a,b)     ((a).s6_laddr[0]  = 0, (a).s6_laddr[1] = 0, \
                                     (a).s6_laddr[2] = 0xFFFF0000, (a).s6_laddr[3]  = (b))
#define IN6_SIT_SET_v4v6(a,b)       ((a).s6_qaddr[0] = 0, (a).s6_laddr[2] = 0, \
                                     (a).s6_laddr[3] = (b))
#define IN6_SIT_v4addr(a)           ((a).s6_laddr[3])
#define IN6_SIT_EQUAL(a,b)  (((a).s6_qaddr[0] == (b).s6_qaddr[0]) && \
			     ((a).s6_laddr[3] == (b).s6_laddr[3]) && \
			     (((a).s6_laddr[2] == (b).s6_laddr[2]) || \
			      ((a).s6_qaddr[0] == 0l && (b).s6_qaddr[0] == 0l && \
			       ((a).s6_laddr[2] == 0xFFFF0000 && (b).s6_laddr[2] == 0) || \
			       ((a).s6_laddr[2] == 0 && (b).s6_laddr[2] == 0xFFFF0000))))
#endif
#else
/* PG_TODO: Do some other time using 32-bit fields */
#endif /* __alpha */

#define	IPV6ADDR_LINKLOCAL_ALL_NODES_INIT { \
	0xFF, IN6_MCSCOPE_LINKLOCAL, 0, 0, \
	0, 0, 0, 0, \
	0, 0, 0, 0, \
	0, 0, 0, 1 \
    }

#define	IPV6ADDR_LINKLOCAL_ALL_ROUTERS_INIT { \
	0xFF, IN6_MCSCOPE_LINKLOCAL, 0, 0, \
	0, 0, 0, 0, \
	0, 0, 0, 0, \
	0, 0, 0, 2 \
    }

#if !defined(_KERNEL) && !defined(KERNEL)
extern const struct in6_addr in6addr_any;
#endif

#define	IN6ADDR_ANY_INIT { \
	0, 0, 0, 0, \
	0, 0, 0, 0, \
	0, 0, 0, 0, \
	0, 0, 0, 0  \
    }

#if !defined(_KERNEL) && !defined(KERNEL)
extern const struct in6_addr in6addr_loopback;
#endif

#define	IN6ADDR_LOOPBACK_INIT { \
	0, 0, 0, 0, \
	0, 0, 0, 0, \
	0, 0, 0, 0, \
	0, 0, 0, 1  \
    }

/*
 * IPv6 multicast scope.
 */
#define	IN6_MCSCOPE_NODELOCAL	0x01
#define	IN6_MCSCOPE_LINKLOCAL	0x02
#define	IN6_MCSCOPE_SITELOCAL	0x05
#define	IN6_MCSCOPE_ORGLOCAL	0x08
#define	IN6_MCSCOPE_GLOBAL	0x0E

#define	IN6_MCFLAG_PERMANENT	0x00
#define	IN6_MCFLAG_TRANSIENT	0x10

#ifdef IN6_HAS_64BIT_INTTYPE
#define	IN6_SET_MULTICAST_CLASS(addrp, class) \
    ((void) (*(u_int64_t *) (addrp) = IN6__MK1_MSB64_VALUE(0xFF, class)))
#else
#define	IN6_SET_MULTICAST_CLASS(addrp, class) \
    ((void) ((addrp)->s6_laddr[0] = IN6__MK1_MSB32_VALUE(0xFF, class), (addrp)->s6_laddr[1] = 0))
#endif

/*
 * Define the socket address for IPv6
 */
struct sockaddr_in6 {
#if     defined(_SOCKADDR_LEN) || defined(_KERNEL)
#define SIN6_LEN
    u_char	    sin6_len;		/* length of this struct */
    u_char	    sin6_family;	/* AF_INET6 */
#else
    u_int16m_t	    sin6_family;	/* AF_INET6 */
#endif
    u_int16m_t	    sin6_port;		/* Transport layer port # */
    u_int32m_t	    sin6_flowlabel;	/* IPv6 flow information */
    struct in6_addr sin6_addr;		/* IPv6 address */
};

/*
 * Define the route structure template. This is required because
 * the standard route structure uses a sockaddr structure, which is
 * no longer large enough to hold IPv6 addresses.
 * XXX This can be removed if the route structure becomes large enough
 */
struct ip6route {
    struct rtentry *ro_rt;
    struct sockaddr_in6 ro_dst;
};


/*
 * Define the structure used for interface ioctls with IPv6 addresses
 */
#ifdef IFNAMSIZ
struct ip6ifreq {
    char ifr6_name[IFNAMSIZ];			/* i/f name, e.g. "ln0" */
    union {
        struct sockaddr		un_addr;	/* address */
        struct sockaddr_in	un_addr4;	/* address */
        struct sockaddr_in6	un_addr6;	/* address */
	struct {
	    caddr_t		un_data;	/* generic pointer... */
	    size_t		un_datalen;	/* generic pointer... */
	} un_1;
	struct {
	    int                 un_value;	/* generic value... */
	    int			un_code;	/* generic ... */
	} un_2;
	struct {
       	    struct in_addr	un_tdst4;	/* tunnel dst IPv4 address */
       	    struct in_addr	un_tsrc4;	/* tunnel src IPv4 address */
	} un_3;
    } ifr6_un;
#define ifr6_addr	ifr6_un.un_addr
#define ifr6_addr4	ifr6_un.un_addr4
#define ifr6_addr6	ifr6_un.un_addr6
#define ifr6_data	ifr6_un.un_1.un_data
#define ifr6_datalen	ifr6_un.un_1.un_datalen
#define ifr6_value	ifr6_un.un_2.un_value
#define ifr6_code	ifr6_un.un_2.un_code
#define ifr6_tdst4	ifr6_un.un_3.un_tdst4
#define ifr6_tsrc4	ifr6_un.un_3.un_tsrc4
    unsigned ifr6_flags;			/* */
#define	IFR6_NODAD		0x0001U		/* suppress duplicate address detection */
#define	IFR6_ADDTOKEN		0x0002U		/* add token to end of prefix */
#define	IFR6_DADFAILED		0x0004U
#define	IFR6_DADINPROGRESS	0x0008U
#define	IFR6_DEPRECATED		0x0010U
#define IFR6_V4TUNNEL		0x0020U		/* IPv4 tunnel */
};

#define SIOCIPV6IFINIT	_IOWR('i', 250, struct ip6ifreq)    /* init if IPv6 */
#define SIOCIPV6AIFADDR	_IOWR('i', 251, struct ip6ifreq)    /* add if IPv6 */
#define SIOCIPV6SIFATTR	_IOW ('i', 252, struct ip6ifreq)    /* set IPv6 if attributes */
#define SIOCIPV6DIFADDR	_IOW ('i', 253, struct ip6ifreq)    /* del if IPv6 */
#define SIOCIPV6GIFATTR	_IOWR('i', 254, struct ip6ifreq)    /* get IPv6 if attributes */
#define SIOCIPV6MIFADDR	_IOWR('i', 255, struct ip6ifreq)    /* mdfy IPv6 addr opts */
#define SIOCIPV6GIFADDR	_IOWR('i', 246, struct ip6ifreq)    /* get IPv6 addr opts */
#endif

/*
 * Define the structure that is added as a control message to
 * incoming raw packets before being handed to the user.
 */
struct rip6_header {
    int   rhdr_ifindex;          /* ifnet index number */
    int   rhdr_pktflags;         /* The mbuf flags (M_MCAST etc) */
    u_short rhdr_offset;         /* data offset in IPv6 packet */
    u_char  rhdr_proto;          /* The protocol type */
    u_char  rhdr_authOK;         /* TRUE if the pkt was authenticated */
};

struct in6_pktinfo {
    int			ipi6_ifindex;	/* interface index */
    u_int		ipi6_hoplimit;	/* IPv6 hop limit */
    struct in6_addr	ipi6_addr;	/* IPv6 address */
};

#endif /* _IN6_H_ */
